
import pickle
###########################
###########################
####DESPIKE Functions######
###########################
###########################
def showspikes(file):
    '''
    shows dictionary of spike values saved in file
    file = path to spike file, without -.spike-
    '''
    with open(file+'.spike', 'rb') as f:
        b = pickle.load(f)
    return print(b)

def removespikes(a,file):
    '''
    removes the spike values for scan a
    a = scan number, where all scans are removed
    file = path to spike file, without -.spike-
    '''
    with open(file+'.spike', 'rb') as f:
        b = pickle.load(f)
    try:
        del(b[a])
        with open(file+'.spike', 'wb') as f:   
            pickle.dump(b,f,protocol=pickle.HIGHEST_PROTOCOL)
    except:
        print('{} not in {}'.format(a,file))

def addspike(a,c,file):
    '''
    adds one  spike value c for scan a
    a = scan number, where all scans are removed
    file = path to spike file, without -.spike-
    '''
    try:
        with open(file+'.spike', 'rb') as f:
            b = pickle.load(f)
    except:
        with open(file+'.spike', 'wb') as f:
            pickle.dump({},f,protocol=pickle.HIGHEST_PROTOCOL)
        with open(file+'.spike', 'rb') as f:
            b = pickle.load(f)
    try:
        if a in b:
            if c not in b[a]:
                b[a].append(c)
        else:
            b[a] = [c]
        with open(file+'.spike', 'wb') as f:   
            pickle.dump(b,f,protocol=pickle.HIGHEST_PROTOCOL)
    except:
        print('oops, something went wrong')




#imports:
import numpy as np
from scipy import interpolate
import matplotlib.pyplot as plt
import pandas
import pickle
#####


###################
####XMCD######
def XMCD(pdat,mdat,ene,det,mon,log):
    '''
    XMCD function:
    pdat = positive lists
    mdat = negative list 
    ene = energy column
    det = detector column
    mon = monitor column
    '''
    t2pdat = pdat
    t2mdat = mdat
    pmin=[]
    pmax=[]
    for n in range(len(t2pdat)):
        pmin.append(np.min(t2pdat[n][ene]))
        pmax.append(np.max(t2pdat[n][ene]))
    mmin=[]
    mmax=[]
    for n in range(len(t2mdat)):
        mmin.append(np.min(t2mdat[n][ene]))
        mmax.append(np.max(t2mdat[n][ene]))
    xmin = np.max(pmin+mmin)
    xmax = np.min(pmax+mmax)
    xx = np.linspace(xmin+0.01,xmax-0.01,10000) # energy interpolation range
    t3pdat=[] # alles interpolierte
    for n in range(len(t2pdat)):
        v1  = np.array(t2pdat[n][ene])
        if mon != False:
            v21 = np.array(t2pdat[n][det])
            v22 = np.array(t2pdat[n][mon])
            if log == True:
                v2  = np.log(v22/v21)
            else:
                v2  = v21/v22
        if mon ==False:
            v2 = np.array(t2pdat[n][det])
        t3pdat.append(interpolate.interp1d(v1,v2)(xx)) 
    t3mdat=[]
    for n in range(len(t2mdat)):
        v1  = np.array(t2mdat[n][ene])
        if mon != False:
            v21 = np.array(t2mdat[n][det])
            v22 = np.array(t2mdat[n][mon])
            v2  = v21/v22
        if mon == False:
            v2 = np.array(t2mdat[n][det])
        t3mdat.append(interpolate.interp1d(v1,v2)(xx))
    t4pdat=[] # all from +hel merged
    for k in range(0,len(xx)):
        t4pdat.append(np.sum([t3pdat[s][k] for s in range(len(t3pdat))])/int(len(t3pdat)))
    t4mdat=[] # all from -hel merged
    for k in range(0,len(xx)):
        t4mdat.append(np.sum([t3mdat[s][k] for s in range(len(t3mdat))])/int(len(t3mdat)))
    t5pdat= np.array(t4pdat)
    t5mdat= np.array(t4mdat)
    pdm = t5pdat/t5mdat    #plus signal divided by minus signal
    #correlation of plus and minus signal
    corrpre = np.poly1d(np.polyfit(xx[:int(len(xx)*0.1)],pdm[:int(len(xx)*0.1 )],1))
    corrpost = np.poly1d(np.polyfit(xx[int(len(xx)-len(xx)*0.1):],pdm[int(len(xx)-len(xx)*0.1):],1))
    x1 = xx[2]
    x2 = xx[-2]
    cfy1 = corrpre(x1) #corr intens. at 
    cfy2 = corrpost(x2)
    #correlated signal with pmerged
    t5mdat = (t5mdat*(cfy1-x1*(cfy1-cfy2)/(x1-x2)+xx*(cfy1-cfy2)/(x1-x2)))
    #####
    xas = (t5pdat+t5mdat)/2 
    #Subtracting backgrounds now:
    linbkg = np.poly1d(np.polyfit(xx[:int(len(xx)*0.1)],xas[:int(len(xx)*0.1 )],1))(xx)
    xas    = xas    - linbkg
    pxas   = t5pdat - linbkg
    mxas   = t5mdat - linbkg
    #### now normalized to XAS maximum
    xmcd = (t5pdat-t5mdat)/np.max(xas)
    return xx, pxas, mxas, xas, xmcd
### 




########### HSYT functions:
###########
####HYST###
def HYST(df,fld, ene, det, mon, Epre, Eedg):
    '''
    HYST : edge/pre_edge
    VEKMAG compatible
    df     = takes pandas data frame
    fld    = column name of field values
    ene    = column name of energy values
    det    = column name of detector values
    mon    = column name of monitor
    Epre    = energy of pre edge point
    Eedg    = energy of edge point
    '''
    df = df
    hyst1  = []
    field1 = []
    for n in np.arange(-6.8,6.8,0.1):
        n   = np.around(n,1)
        t1  = df[np.around(df[ene],1) == Eedg]
        t1  = t1[np.around(t1[fld],1) == n]
        t11 = np.mean(t1[det]/t1[mon])
        t1  = df[np.around(df[ene],1) == Epre]
        t1  = t1[np.around(t1[fld],1) == n]
        t22 = np.mean(t1[det]/t1[mon])
        hyst1.append(t11/t22)
        field1.append(n)
    ############
    return field1,hyst1


###########
####HYST###
def HYST2(df,fld, ene, det, mon, Epre, Eedg):
    '''
    HYST2 : (edge - pre_edge)/pre_edge
    VEKMAG compatible
    df     = takes pandas data frame
    fld    = column name of field values
    ene    = column name of energy values
    det    = column name of detector values
    mon    = column name of monitor
    Epre    = energy of pre edge point
    Eedg    = energy of edge point
    '''
    df = df
    hyst1  = []
    field1 = []
    for n in np.arange(-6.8,6.8,0.1):
        n   = np.around(n,1)
        t1  = df[np.around(df[ene],1) == Eedg]
        t1  = t1[np.around(t1[fld],1) == n]
        t11 = np.mean(t1[det]/t1[mon])
        t1  = df[np.around(df[ene],1) == Epre]
        t1  = t1[np.around(t1[fld],1) == n]
        t22 = np.mean(t1[det]/t1[mon])
        hyst1.append((t11 - t22)/t22)
        field1.append(n)
    ############
    return field1,hyst1

def HYST3en(df,fld, ene, det, mon, Epre, Eedg1, Eedg2):
    '''
    HYST : edge/pre_edge
    VEKMAG compatible
    df     = takes pandas data frame
    fld    = column name of field values
    ene    = column name of energy values
    det    = column name of detector values
    mon    = column name of monitor
    Epre    = energy of pre edge point
    Eedg1   = energy of edge point 1
    Eedg2   = energy of edge point 2 
    '''
    df = df
    hyst1  = []
    hyst2  = []
    field1 = []
    for n in np.arange(-6.8,6.8,0.1):
        n   = np.around(n,1)
        t1  = df[np.around(df[ene],1) == Eedg1]
        t1  = t1[np.around(t1[fld],1) == n]
        t11 = np.mean(t1[det]/t1[mon])
        t2  = df[np.around(df[ene],1) == Eedg2]
        t2  = t2[np.around(t2[fld],1) == n]
        t12 = np.mean(t2[det]/t2[mon])
        t1  = df[np.around(df[ene],1) == Epre]
        t1  = t1[np.around(t1[fld],1) == n]
        t22 = np.mean(t1[det]/t1[mon])
        hyst1.append(t11/t22)
        hyst2.append(t12/t22)
        field1.append(n)
    ############
    return field1,hyst1,hyst2




##########sumrules functions##
def sumrules(xx,xas00, xmcd, px,nh):
    '''
    xx = global energy scale
    xas00 = xas - stepfunction
    xmcd = xmcd
    px = point in energy for p value
    nh = number of electron holes'''
    r=np.max(np.cumsum(xas00))
    q=np.max(np.cumsum(xmcd))
    p=np.cumsum(xmcd)[list(np.around(xx,2)).index(px)]
#print(np.max(np.cumsum(np.linspace(0,1,steps)))/steps)
    lz=nh*2*q/(3*r)
    sz=nh*(3*p-2*q)/(2*r)
    return sz,lz

def LDS(xx, xmcd, px):
    '''
    xx = global energy scale
    xmcd = xmcd
    px = point in energy for p value'''
    q=np.absolute(np.cumsum(xmcd))[-1]
    p=np.absolute(np.cumsum(xmcd)[list(np.around(xx,2)).index(px)])
    #print(q,p)
    lds=(4/3)*q/(3*p-2*q)
    return lds


from IPython.core.display import display, HTML


def toggle(a):
    '''
    toggles cell in Jupyter notebook;
    makes button with label 'a' 
    '''
    a = a
    toggle_str = '''
        <form action="javascript:code_toggle()"><input type="submit" id="toggleButton" value="{}"></form>
        '''.format(a)

    toggle_prepare_str = '''
        <script>
        function code_toggle() {
            if ($('div.cell.code_cell.rendered.selected div.input').css('display')!='none'){
                $('div.cell.code_cell.rendered.selected div.input').hide();
            } else {
                $('div.cell.code_cell.rendered.selected div.input').show();
            }
        }
        </script>
    '''
    display(HTML(toggle_prepare_str + toggle_str))
    def toggle_code():
        display(HTML(toggle_str))



def showspeccom(a):
    '''
    a = SPEC file
    returns list with commands/comments #C and #S
    '''
    clst=[]
    with open(a,'r')as f:
        for line in f:
            if '#C' in line:
                clst.append(line.replace('\n',''))
            if '#S' in line:
                clst.append(line.replace('\n',''))
    return clst


#import:
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt


class PiMoss:
    '''
    ---under construction---
    class to create Mossbauer measurement class from pi software package: 
    https://www.uni-due.de/~hm236ap/hoersten/home.html
    takes path to .dat file. do not use .dat extension in name
    '''
    
    def __init__(self,path):
        def impspec(a):
            '''
            function to import the plot data as a pandas DataFrame
            '''
            return pd.read_csv(a, sep='\t',engine='python',skiprows=1,
                            na_values='-',header=None)        
        try:
            self.df   = impspec(path + '.dat')
            self.v    = np.array(self.df[0])  #returns velocity as numpy array
            self.m    = np.array(self.df[1])  #returns measurement as numpy array
            self.err  = np.array(self.df[2])  #returns velocity as numpy array
            self.lsfit= []                    #list of np.arrays for the fit models
            for n in range(len(self.df.columns)-3):
                self.lsfit.append(np.array(self.df[n+3]))
        except:
            raise IOError('File not found. Make sure it is a .dat-file')

    def plot(self,kind='all'):
        '''
        fast plotting functions functions:
        input string: 
        'all'        errorbar measurement + all fit models
        'err'        errorbar measurement
        'noerr'      measurement no errorbar
        'allnoerr'   all fit models + measurement no errorbar
        '''
    
        plotlist = ['all','err','noerr','allnoerr']
        
        if kind not in plotlist:
            raise ValueError('Plot option not in list. Please choose from {}'.format(plotlist))
        if kind == 'all':
            plt.errorbar(self.v,self.m,self.err)
            for n in self.lsfit:
                plt.plot(self.v,n)
        if kind == 'err':
            plt.errorbar(self.v,self.m,self.err)
        if kind == 'noerr':
            plt.plot(self.v,self.m)
        if kind == 'allnoerr':
            plt.plot(self.v,self.m)
            for n in self.lsfit:
                plt.plot(self.v,n)
        plt.xlabel('velocity [mm/s]'),plt.ylabel('absorption [arb. units]')
 



#imports:
import pandas as pd 
#####

####readin functions for different theory/simulation programs:

def multix(a):
    '''
    import multiX output file 
    returns panda file with energy, imaginary part that simulates the xas, and the real part
    '''
    return pd.read(a,header=['energy','img','r'],delim_whitespace=True)

def crispy(a):
    return pd.read_csv(a,delim_whitespace=True,comment='#', header=3)




#imports:
import numpy as np
from scipy import interpolate
import matplotlib.pyplot as plt
import pandas as pd 
import pickle
#####

####readin functions from different beamlines:
##VEKMAG### BESSY2
### define get header function from large, unsplit and file
def VEKgetheader(b,file): #its written that b is scan number! 
    r = open(file,'r')
    mylist = r.read().splitlines() 
    matching = [p for p in mylist if "#L" in p]
    scannr =   [p for p in mylist if "#S" in p]
    r.close()
    rscan = []
    for n in scannr:
        v = n.split()[1]
        rscan.append(int(v))
    header = []
    for n in range(len(rscan)):
        t1 = [p for p in matching[n].split() if not '#L' in p]
        for n in range(len(t1)):
            t1[n] = '{}_'.format(n+1)+t1[n]
        header.append(t1)
    c = rscan.index(b)
    return header[c]

def VEKgetcommand(b,file):
    r = open(file,'r')
    mylist = r.read().splitlines() 
    scannr =   [p for p in mylist if "#S" in p]
    r.close()
    rscan = []
    comma = []
    for n in scannr:
        v = n.split()[1]
        w = ' '.join(n.split()[2:])
        rscan.append(int(v))
        comma.append(w)
    c = rscan.index(b)
    return comma[c]


#function for read in a is number of scan
def VEKreadin(a,file,raw=False):
    a = a
    dff = pd.read_csv(file+'_{}.dat'.format(a), delim_whitespace=True,header=0,names=getheader(a,file),comment='#')
    if raw==False:
        try:
            with open(file+'.spike','rb') as f:
                b = pickle.load(f)
                if a in b:
                    todrop = b[a]
                    dff = dff.drop(todrop)
        except:
            pass
    else: 
        pass
    return dff





#imports:
import numpy as np
from scipy import interpolate
import matplotlib.pyplot as plt
import pandas
import pickle
#####



def linprebkg(xx,xas,x1,x2):
    '''
    xx = global energy scale
    xas = xas
    use from x1 to x2 for cutting background region'''
    return np.poly1d(np.polyfit(xx[list(np.around(xx,2)).index(x1):list(np.around(xx,2)).index(x2)], xas[list(np.around(xx,2)).index(x1):list(np.around(xx,2)).index(x2)],1))(xx)



def norm01(xx,xas,energy1,a1): #a1 nimmt list object norm to one at energy1
    faktor = 1/xas[list(np.around(xx,2)).index(energy1)]
    return a1*faktor



def smooth(y, box_pts):
    '''
    smoothing function
    y =  function to smooth
    box_pts = points to average 
    '''
    box = np.ones(box_pts)/box_pts
    y_smooth = np.convolve(y, box, mode='same')
    return y_smooth

#step function for branching ratio
def step(a,tf,xp):
    return a*(1/(1+np.exp(tf - xp)))

    
###index function
def idx(a1, b1):
    '''find index of value a1 in list b1'''
    return list(np.around(b1,2)).index(a1)
